\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.

default(strictargs,1);
default(recover,0);
read("../a073057-code.gp");
read("my-oeis.gp");

check_count = 0;
check_equal(got,want,name="") =
{
  check_count++;
  if(got != want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}


\\-----------------------------------------------------------------------------
\\ LtoH Full

LtoH_state_table_full = {[
  \\ generated by tools/gen-states-gp.pl full
  1,2;      \\  1   2a -> 2a,3a
  5,14;     \\  2   3a -> 2b,3c
  8,11;     \\  3   4a -> 1b,4c
  4,3;      \\  4   1a -> 1a,4a
  1,9;      \\  5   2b -> 2a,2c
  8,11;     \\  6   3g -> 1b,4c
  18,10;    \\  7   4g -> 3d,3b
  4,12;     \\  8   1b -> 1a,1c
  5,14;     \\  9   2c -> 2b,3c
  5,13;     \\ 10   3b -> 2b,2d
  19,15;    \\ 11   4c -> 4d,4b
  8,11;     \\ 12   1c -> 1b,4c
  18,10;    \\ 13   2d -> 3d,3b
  18,10;    \\ 14   3c -> 3d,3b
  8,16;     \\ 15   4b -> 1b,1d
  19,15;    \\ 16   1d -> 4d,4b
  5,13;     \\ 17   2e -> 2b,2d
  26,22;    \\ 18   3d -> 3f,3e
  23,27;    \\ 19   4d -> 4f,4e
  8,16;     \\ 20   1e -> 1b,1d
  18,17;    \\ 21   2f -> 3d,2e
  18,17;    \\ 22   3e -> 3d,2e
  23,24;    \\ 23   4f -> 4f,1f
  19,20;    \\ 24   1f -> 4d,1e
  0,0;      \\ 25 unused
  26,21;    \\ 26   3f -> 3f,2f
  19,20;    \\ 27   4e -> 4d,1e
  4,6;      \\ 28   T -> 1a,3g
  5,7;      \\ 29   U -> 2b,4g
  28,29     \\ 30   S -> T,U
  \\ end generated
]};

{
  my(len=matsize(LtoH_state_table)[1]);
  check_equal(LtoH_state_table_full[1..len,], LtoH_state_table);
}

b_by_LtoH_using_M_came_from(m) =
{
  my(state=30); \\ state S
  while(m>=4,
        my(bit);[m,bit] = m_came_from_and_bit(m);
        state=LtoH_state_table_full[state,bit+1]);
  state=LtoH_state_table_full[state,bittest(m,0)+1];  \\ second highest
  state=LtoH_state_table_full[state,bittest(m,1)+1];  \\ highest
  state%4 + 1;
}
{
  my(v=a_vector(7));
  check_equal(vector(#v,m,m--; b_by_LtoH_using_M_came_from(m)),
              v);
}


\\-----------------------------------------------------------------------------
\\ Logs Over-Estimate

\\ log(m)/log(3) = log(m)/log(2) * log(3)/log(2)
\\ log(3)/log(2)   \\ 1.58
\\ log(2)/log(3)   \\ 0.6309
\\ log(m) < (L+1) * log(2)
\\ log(m) *log( < (L+1) * log(2)
\\ 5/8.0 == 0.625  too small
\\ 21/32.0 == 0.65625
\\ my(L=log(2)/log(3),best=oo); print(L); \
\\ for(d=2,100, \
\\   my(n=ceil(d*L)); \
\\   n/d >= L || error(); \
\\   if(n/d-L<best, \
\\     print(n/d" ",n/d+.0" "n/d-L); \
\\     best=n/d-L));

k_over_estimate(n) = logint(n,2)*12\19+1;
{
  for(n=1,3^8,
     my(k=k_over_estimate(n));
     k==floor(k) || error();
     n<3^(k+1) || error("k_over_estimate() n=",n));
}



\\-----------------------------------------------------------------------------
\\ Alternate Paperfolding Directions

\\            1  2   3   4
term_to_dz = [1, I, -I, -1];

terms_to_points(v) =
{
  my(ret=vector(#v+1));
  for(i=1,#v, ret[i+1] = ret[i] + term_to_dz[v[i]]);
  ret;
}
{
  my(v=[1,2,3,4]);
  check_equal(terms_to_points(v),
              [0, 1, 1+I, 1, 0],
              "points initial L");

  v=morphism_expand_vector(v);
  check_equal(terms_to_points(v),
              [0, 1, 1+I, 2+I, 2, 1, 1+I, I, 0],
              "points expand, figure of 8");
}


\\-----------------------------------------------------------------------------
\\ GRS

\\ Count11Pairs(n) = hammingweight(bitand(n,n>>1));
\\ A014081(n) = Count11Pairs(n);
\\ {
\\   my(v=OEIS_samples("A014081"));  \\ OFFSET=0
\\   check_equal(vector(#v,n,n--; A014081(n)), v,
\\               "Count11Pairs() A014081 samples");
\\ }
\\ 
\\ GRS(n) =
\\ {
\\   n>=0 || error("GRS() is for n>=0");
\\   (-1)^Count11Pairs(n);
\\ }
\\ {
\\   my(v=OEIS_samples("A020985"));
\\   check_equal(vector(#v,n,n--; GRS(n)), v,
\\               "GRS() A020985 samples");
\\ }


\\-----------------------------------------------------------------------------
\\ Morphism A100260+1

\\ {
\\   my(v=OEIS_samples("A100260"));
\\   check_equal(vector(#v,n, A100260(n)), v,
\\               "A100260() vs samples");
\\ }
\\ 
\\ morphism_term(n) =
\\ {
\\   n>=1 || error();
\\   A100260(n) + 1;
\\ }
\\ {
\\   my(morphism_to_GRS=[+1,+1,-1,-1]);
\\   check_equal(vector(2^10,n,n--; GRS(n)),
\\               vector(2^10,n, morphism_to_GRS[morphism_term(n)]),
\\               "GRS() from morphism_term()");
\\ }


\\-----------------------------------------------------------------------------
\\ b_by_recurrence(m) = A073057

{
  my(v=OEIS_samples("A073057"));
  check_equal(vector(#v,m,m--; b_by_recurrence(m)), v,
              "b_by_recurrence(m) vs A073057 samples");

  v=OEIS_bfile_samples("A073057");
  check_equal(vector(#v,m,m--; b_by_recurrence(m)), v,
              Str("b_by_recurrence(m) vs b073057.txt (size ",#v,")"));
}


\\-----------------------------------------------------------------------------
check_equal(m,'m);
check_equal(mdash,'mdash);
check_equal(i,'i);
check_equal(h,'h);
check_equal(k,'k);
check_equal(n,'n);
check_equal(p,'p);
check_equal(t,'t);
check_equal(ret,'ret);
print("end");
