\\ Copyright 2021 Kevin Ryde
\\ 
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\ 
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\ 
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


default(strictargs,1);
default(recover,0);
read("../a073057-code.gp");

check_count = 0;
check_equal(got,want,name="") =
{
  check_count++;
  if(got != want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}


\\-----------------------------------------------------------------------------


{
  my(kv=[8,6,3,1],
     r =from_binary(101101),
     m=kvr_to_m(kv,r),
     p=2,
     width=16);
  my(kvr=m_to_kvr(m)); kvr==[kv,r] || error();
  print(m" "to_binary(m));
  while(m >= 2^p,
        m>=0 || error();
        my(k=m_came_from_k(m>>(p-2)));
        printf("   %*s\n", width, to_binary(m));
        printf(" - %*s%*s   %d*3^%d\n",
               width-p, to_binary(3^k),
               p, "",
               1<<p, k);
        printf("   %s\n",
               concat(vector(width,i,"-")));
        m-=3^k<<p;
        p++);
  printf("   %*s   r\n", width, to_binary(m));
  m==r || error();
  quit;
}




\\-----------------------------------------------------------------------------
\\ Morphism, A100260, GRS4
\\
\\ The morphism applied repeatedly to a starting string 1 gives A100260+1.
\\ A100260 is a way to calculate terms of the Golay-Rudin-Shapiro sequence
\\ (A020985), and gives directions of segments in the alternate paperfolding
\\ curve (more on that below).

\\ A100260 = morphism 0 -> 01, 1 -> 02, 2 -> 31, 3 -> 32
\\ which is same as morphism_table[] but 0..3 instead of 1..4.
\\
A100260(n) =
{
  n--;
  my(s=1);  \\ calculate as terms 1..4
  forstep(i=if(n,logint(n,2)),0,-1,
     s=morphism_bit(s,bittest(n,i)));
  s-1;      \\ then final A100260 is 0..3
}
{
  print1("cf A100260 = ");
  for(n=1,16, print1(A100260(n),","));
  print("...  (continuing infinitely)");

  my(v=morphism_vector(10));
  vector(#v,n, A100260(n)+1) == v   || error();

  my(A100260_data=[0,1,0,2,0,1,3,1,0,1,0,2,3,2,0,2,0,1,0,2,0,1,3,1,
                   3,2,3,1,0,1,3,1,0,1,0,2,0,1,3,1,0,1,0,2,3,2,0,2,
                   3,2,3,1,3,2,0,2,0,1,0,2,3,2,0,2,0,1,0,2,0,1,3,1]);
  vector(#A100260_data,n, A100260(n)) == A100260_data || error();
}

\\ The Golay-Rudin-Shapiro sequence is +1 or -1 according as the parity of
\\ the number of 11 bit pairs in m (including 11 pairs overlapping), for
\\ m>=0.
\\
Count11Pairs(m) = hammingweight(bitand(m,m>>1));   \\ A014081
GRS(m) = (-1)^Count11Pairs(m);

\\ The morphism is related
\\
\\          1,  2,  3,  4            morphism
\\     ->  +1, +1, -1, -1            GRS
{
  my(v=morphism_vector(10));
  vector(#v,m,m--; GRS(m)) == apply(s->[+1,+1,-1,-1][s], v)  || error();
}

\\ The parity of 11 bit pairs can be counted by examining the bits of m and
\\ keeping track of the parity seen so far, and whether he preceding bit was
\\ 0 or 1.  A 1 bit after a preceding 1 means flip parity.
\\
\\ In the morphism, the preceding bit is the "first" or "second" term of
\\ each expansion,
\\
\\     s=1 or s=4     preceding bit was 0
\\     s=2 or s=3     preceding bit was 1
\\
\\ And the parity of 11 pairs is as per the morphism go GRS mapping above:
\\ s=1 or s=2 is even 11 pairs.  Notice for instance in s=2 or s=3 which is
\\ preceding 1, another flips to s=3 or s=2 respectively.






\\ In the "came from" it can be noticed that m=12,13, which are the start of
\\ the next expansion, are expanding the first term m=0,
\\
\\     m =   4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21 ...
\\     from  0  0  1  1  2  2  3  3  0  0  1  1  2  2  3  3  4  4 ...
\\
\\ The number of "came from" needed to reach m=0..3 is a kind of depth.
\\ The start of each expanded+appended part is always depth 1 and the end of
\\ expansion level k is depth k.
\\
\\ b_recurrence_depth(m) =
\\ {
\\   my(ret=0);
\\   while(m >= 4,
\\         my(bit); [m,bit] = m_came_from_and_bit(m); ret++);
\\   ret;
\\ }
\\ {
\\   print1("depth  ");
\\   my(k=0);
\\   for(m=0,4*3^2+4,
\\      if(m==len(k), print1(" ");k++);
\\      printf("%d", b_recurrence_depth(m)));
\\   print("...");
\\ }


  \\ my(v=equality_places(10));
  \\ vector(#v,n,n--; 4*A006046(n)) == v     || error();

\\ In A073057, this means that the
\\ only places terms 1,2,3,4 are exactly in proportions 1:1:1:1 is at the
\\ end of some expansion the original 1,2,3,4.
\\
\\ If A073057 level k contains a position E where the first E terms have
\\ 1:1:1:1, then the expand and concat means level k+1 contains equality at
\\ additionally
\\
\\     len(k) + 2*E      expanded and appended to len(k)
\\     = 4*3^k + 2*E
\\
\\ So
\\
\\     within k=0:  0            <-- initial equality at no terms
\\     within k=1:  4
\\     within k=2:  12, 20
\\     within k=3:  36, 44, 60, 76,
\\     = 4*A006046


\\-----------------------------------------------------------------------------
check_equal(m,'m);
check_equal(mdash,'mdash);
check_equal(i,'i);
check_equal(h,'h);
check_equal(k,'k);
check_equal(n,'n);
check_equal(p,'p);
check_equal(t,'t);
check_equal(ret,'ret);
print("end");
